import sys
import requests

from toledo.utils import SESSION, get_saml2_csrf
from toledo.utils import get_saml2_relay_and_request
from toledo.utils import get_saml2_relay_and_response
from toledo.utils import get_saml2_session_ss

from toledo.utils import post_saml2_csrf
from toledo.utils import post_saml2_relay_and_request
from toledo.utils import post_saml2_relay_and_response

from toledo.utils import get_schedule_call_url
from toledo.utils import get_schedule_manifest
from toledo.utils import get_start_html
from toledo.utils import get_start_up_info
from toledo.utils import get_x_csrf_token

from toledo.api import ToledoApi

from toledo.utils import SESSION


class KULoketLogin:

    def __init__(self, portal_session: requests.Session) -> None:

        self._SESSION = portal_session

    def __call__(self) -> requests.Session:

        try:

            # 3. KULoket Schedule

            SESSION = self._SESSION

            if '_shibsession_' not in str(SESSION.cookies.get_dict()):

                raise AttributeError

            # Get KULoket Schedule URL
            schedule_url = ToledoApi(
                session=SESSION).get_kuleuven_schedule_url()

            # Load KULoket Schedule
            schedule_html = get_start_html(
                url=schedule_url
            )

            # Get RelayState & SAMLRequest
            relaystate_samlrequest = get_saml2_relay_and_request(
                html=schedule_html
            )

            # Post RelayState & SAMLRequest
            first_csrf_html = post_saml2_relay_and_request(
                post_info=relaystate_samlrequest
            )

            # Get first csrf
            first_csrf = get_saml2_csrf(
                html=first_csrf_html
            )

            # Get session_ss
            session_ss = get_saml2_session_ss(
                html=first_csrf_html
            )

            # Post first csrf
            first_relaystate_first_samlresponse_html = post_saml2_csrf(
                post_info=first_csrf,
                session_ss=session_ss
            )

            # Get first RelayState & first SAMLResponse
            first_relaystate_first_samlresponse = get_saml2_relay_and_response(
                html=first_relaystate_first_samlresponse_html
            )

            # Post first RelayState & first SAMLResponse
            second_relaystate_second_samlresponse_html = post_saml2_relay_and_response(
                post_info=first_relaystate_first_samlresponse
            )

            # Get second RelayState & second SAMLResponse
            second_relaystate_second_samlresponse = get_saml2_relay_and_response(
                second_relaystate_second_samlresponse_html
            )

            # Post second RelayState & second SAMLResponse
            start_up_html = post_saml2_relay_and_response(
                post_info=second_relaystate_second_samlresponse
            )

            # Get Start_up URL
            start_up_info = get_start_up_info(
                html=start_up_html
            )

            # Get schedule manifest_url
            schedule_manifest_url = get_schedule_manifest(
                url_info=start_up_info,
                schedule_url=schedule_url
            )

            # Get schedule call url
            schedule_call_url = get_schedule_call_url(
                url=schedule_manifest_url
            )

            # Get x-csrf-token and add to headers
            get_x_csrf_token(
                url=schedule_call_url
            )

            return SESSION

        except requests.exceptions.HTTPError as errh:
            sys.exit("Http Error:", errh)
        except requests.exceptions.ConnectionError as errc:
            sys.exit("Error Connecting:", errc)
        except requests.exceptions.Timeout as errt:
            sys.exit("Timeout Error:", errt)
        except requests.exceptions.RequestException as err:
            sys.exit("OOps: Something Else:", err)
        except AttributeError:
            sys.exit(
                'Unable to find _shibsession_ in session. You can only extend portal sessions!')
        except Exception as ex:
            sys.exit(ex)


def extend_session(portal_session: requests.Session) -> requests.Session:

    return KULoketLogin(
        portal_session=portal_session
    )()



import time
import requests
import sys
import os
import yaml
import asyncio

from toledo.utils import set_user_agent
from toledo.utils import clear_session

from toledo.utils import get_start_html
from toledo.utils import get_saml2_relay_and_request
from toledo.utils import get_saml2_relay_and_response
from toledo.utils import get_saml2_csrf
from toledo.utils import get_data_account_id
from toledo.utils import get_websocket_payload
from toledo.utils import next_auth_provoke

from toledo.utils import post_saml2_relay_and_request
from toledo.utils import post_saml2_relay_and_response
from toledo.utils import post_saml2_csrf
from toledo.utils import post_saml2_credentials
from toledo.utils import get_shib_idp_session_ss
from toledo.utils import check_login_state

from toledo.utils import SESSION


class PortalLogin:

    # This class retrieves the _shibsession_ token

    def __init__(self, user: str, password: str) -> None:

        try:

            with open(os.path.join(os.path.dirname(__file__), 'config.yaml'), 'r') as f:
                parser = yaml.safe_load(f)

        except FileNotFoundError:

            sys.exit('Unable to find find config.yaml')

        self._PORTALURL = parser['LOGIN']['PortalURL']

        self._AUTHORIZATION_ENDPOINT = parser['LOGIN']['AuthorizationEndpoint']
        self._TOKEN_ENDPOINT = parser['LOGIN']['TokenEndpoint']
        self._BROKER_ENDPOINT = parser['LOGIN']['BrokerEndpoint']

        self._USER = user
        self._PASSWORD = password
        self._USER_AGENT = parser['USER']['UserAgent']

    def __call__(self) -> requests.Session:

        try:

            if '' in (self._USER, self._PASSWORD):

                raise Exception('Please check your credentials!')

            # 1. Toledo Portal

            # Clear session
            clear_session()

            # Set User-Agent
            set_user_agent(
                user_agent=self._USER_AGENT
            )
            
            # Load Toledo Portal
            portal_html = get_start_html(
                url=self._PORTALURL
            )
            
            # Get RelayState & SAMLRequest
            relaystate_samlrequest = get_saml2_relay_and_request(
                html=portal_html
            )
            
            # Post RelayState & SAMLRequest
            first_csrf_html = post_saml2_relay_and_request(
                post_info=relaystate_samlrequest
            )
            
            # Get first csrf token
            first_csrf = get_saml2_csrf(
                html=first_csrf_html
            )

            # Post first csrf token
            second_csrf_html = post_saml2_csrf(
                post_info=first_csrf
            )

            # Get second csrf token
            second_csrf = get_saml2_csrf(
                html=second_csrf_html
            )
            
            # Post credentials and second csrf token
            third_csrf_html = post_saml2_credentials(
                post_info=second_csrf,
                username=self._USER,
                password=self._PASSWORD
            )
            
            # Check login state
            if not check_login_state(html=third_csrf_html):
                raise Exception('Invalid credentials')
            
            # Get data-account-id (nextauthAccountId)
            data_account_id_info = get_data_account_id(
                html=third_csrf_html
            )

            # Get third csrf token
            third_csrf = get_saml2_csrf(
                html=third_csrf_html
            )

            # Get WebSocket Payload
            websocket_payload_data = get_websocket_payload(
                html=third_csrf_html
            )

            # Get fourth csrf token
            fourth_csrf = asyncio.run(next_auth_provoke(
                payload_data=websocket_payload_data,
                data_account_id_info=data_account_id_info,
                post_info=third_csrf
            ))

            # Post fourth csrf token
            fifth_csrf_html = post_saml2_csrf(
                post_info=fourth_csrf
            )

            # Get fifth csrf token
            fifth_csrf = get_saml2_csrf(
                html=fifth_csrf_html
            )

            # Get shib_idp_session_ss
            shib_idp_session_ss = get_shib_idp_session_ss(html=fifth_csrf_html)

            # Post fifth csrf token
            relaystate_samlresponse_html = post_saml2_csrf(
                post_info=fifth_csrf
            )

            # Get RelayState & SAMLResponse
            relaystate_samlresponse = get_saml2_relay_and_response(
                html=relaystate_samlresponse_html
            )

            # Post RelayState & SAMLResponse
            post_saml2_relay_and_response(
                post_info=relaystate_samlresponse
            )
            
            # At this point we have obtained the shibsession token
            return SESSION

        except requests.exceptions.HTTPError as errh:
            sys.exit("Http Error:", errh)
        except requests.exceptions.ConnectionError as errc:
            sys.exit("Error Connecting:", errc)
        except requests.exceptions.Timeout as errt:
            sys.exit("Timeout Error:", errt)
        except requests.exceptions.RequestException as err:
            sys.exit("OOps: Something Else: ", err)
        except Exception as ex:
            sys.exit(ex)


def create_session_object(user: str, password: str) -> requests.Session:

    return PortalLogin(
        user=user,
        password=password
    )()



# __main__.py

import argparse
import sys
import json

from toledo.portal import create_session_object
from toledo.dashboard import extend_session as dashboard_extend_session
from toledo.kuloket import extend_session as kuloket_extend_session
from toledo.api import create_api_object


def main(args):

    session = create_session_object(
        user=args.rnumber,
        password=args.password
    )
    
    session = dashboard_extend_session(
        portal_session=session
    )

    session = kuloket_extend_session(
        portal_session=session
    )

    api = create_api_object(
        session=session
    )

    if args.enrollments:

        output = api.get_enrollments()

    elif args.todo:

        output = api.get_to_do(
            type=args.todo
        )

    elif args.upcoming:

        output = api.get_upcoming()

    elif args.events:

        output = api.get_events(
            type=args.events
        )

    elif args.schedule:

        output = api.get_schedule()

    if not args.silent:

        json.dump(output, sys.stdout)


if __name__ == '__main__':

    parser = argparse.ArgumentParser(
        epilog='', usage=argparse.SUPPRESS)

    optionsgroup = parser.add_mutually_exclusive_group(required=True)

    optionsgroup.add_argument(
        '--enrollments', '-en', help='retrieve all your enrollments', action='store_true')
    optionsgroup.add_argument(
        '--todo', '-td', help='retrieve your to-do list (tasks or tests)', choices=['task', 'test'])
    optionsgroup.add_argument(
        '--upcoming', '-up', help='retrieve your upcoming courses', action='store_true')
    optionsgroup.add_argument(
        '--events', '-ev', help='retrieve your recent events (messages or updates)', choices=['message', 'update'])
    optionsgroup.add_argument(
        '--schedule', '-sc', help='retrieve your schedule', action='store_true')

    parser.add_argument(
        '--rnumber', '-rn', help='your personal rnumber', nargs=1, required=True)
    parser.add_argument(
        '--password', '-pw', help='your password', nargs=1, required=True)

    parser.add_argument(
        '--silent', '-s', help='surpress output', action='store_true')
    parser.parse_args()
 
    main(parser.parse_args())



import sys
import json
import os
import uuid

try:

    import requests
    import yaml

except ImportError:

    sys.exit('Unable to import some modules')


class ToledoApi:

    def __init__(self, session: requests.Session) -> None:

        try:

            with open(os.path.join(os.path.dirname(__file__), 'config.yaml'), 'r') as f:
                parser = yaml.safe_load(f)
                f.close()

            if '_shibsession_' not in str(session.cookies.get_dict()):

                raise AttributeError

            self._UPCOMING_URL = parser['API']['UpcomingEndpoint']
            self._ENROLLMENTS_URL = parser['API']['EnrollmentsEndpoint']
            self._EVENTS_URL = parser['API']['EventsEndpoint']
            self._TODO_URL = parser['API']['TodoEndpoint']
            self._LINK_URL = parser['API']['LinkEndpoint']
            self._SCHEDULE_URL = parser['API']['ScheduleEndpoint']

            self._TASK = parser['API_TO_DO']['Task']
            self._TEST = parser['API_TO_DO']['Test']
            self._VARIOUS = parser['API_TO_DO']['Various']

            self._SESSION = session

        except FileNotFoundError:
            sys.exit('API: Unable to find find config.yaml!')

        except AttributeError as ex:
            sys.exit(
                'API: Unable to find _shibsession_ in session. Make sure to use a portal or extended portal session!')
        except Exception:
            sys.exit('API: Failed to create API object!')

    ''' PORTAL METHODS '''

    def get_events(self, type: str) -> json:

        try:

            if '_shibsession_' not in str(self._SESSION.cookies.get_dict()):

                raise AttributeError(
                    'You need an api object with portal session cookies to use this method!')

            r = self._SESSION.get(
                url=self._EVENTS_URL
            )

            r.raise_for_status()

            events = r.json()

            if type == 'message':

                return [event for event in events if event['eventType'] == 'message']
            elif type == 'update':

                return [event for event in events if event['eventType'] == 'update']
            else:

                raise Exception('Unsupported event type!')

        except AttributeError as ex:

            sys.exit(f'EVENTS: {ex}')

        except Exception as ex:

            sys.exit(f'EVENTS: {ex}')

    def get_enrollments(self) -> json:

        try:

            if '_shibsession_' not in str(self._SESSION.cookies.get_dict()):

                raise AttributeError(
                    'You need an api object with portal session cookies to use this method!')

            r = self._SESSION.get(
                url=self._ENROLLMENTS_URL
            )

            r.raise_for_status()

            return r.json()

        except AttributeError as ex:

            sys.exit(f'ENROLLMENTS: {ex}')

        except Exception as ex:

            sys.exit(f'ENROLLMENTS: {ex}')

    def get_upcoming(self) -> json:

        try:

            if '_shibsession_' not in str(self._SESSION.cookies.get_dict()):

                raise AttributeError(
                    'You need an api object with portal session cookies to use this method!')

            r = self._SESSION.get(
                url=self._UPCOMING_URL
            )

            r.raise_for_status()

            return r.json()

        except AttributeError as ex:

            sys.exit(f'UPCOMING: {ex}')

        except Exception as ex:

            sys.exit(f'UPCOMING: {ex}')

    def get_kuleuven_schedule_url(self) -> str:

        try:

            r = self._SESSION.get(
                url=self._LINK_URL,
                headers={
                    'Toledo-Language': 'nl'
                }
            )

            r.raise_for_status()

            links = r.json()

            kuloket = [item for item in links if item['label'] == 'KULOKET'][0]

            return [link['url'] for link in kuloket['links'] if link['label'] == 'KULOKET-UURROOSTER'][0]

        except Exception as ex:

            sys.exit(f'SCHEDULE URL: {ex}')

    ''' DASHBOARD METHODS '''

    def get_to_do(self, type: str) -> json:

        try:

            if '_shibsession_' not in str(self._SESSION.cookies.get_dict()):

                raise AttributeError(
                    'You need an api object with portal session cookies to use this method!')

            # Test for auth header
            self._SESSION.headers['Authorization']

            r = self._SESSION.get(
                url=self._TODO_URL
            )

            r.raise_for_status()

            todolist = r.json()

            if type == 'task':

                contenttype = self._TASK

            elif type == 'test':

                contenttype = self._TEST

            else:

                raise Exception('Unsupported todo type!')

            return [item for item in todolist if item['contentInfo']['contentType'] == contenttype]

        except KeyError:

            sys.exit(
                f'TODO: You need an api object with an extended portal session (dashboard) to use this method')

        except AttributeError as ex:

            sys.exit(f'TODO: {ex}')

        except Exception as ex:

            sys.exit(f'TODO: {ex}')

    ''' KULOKET METHODS '''

    def get_schedule(self) -> json:

        try:

            # Test for x-csrf-token header
            self._SESSION.headers['x-csrf-token']

            # Generate smal uuid
            batch_uuid = str(uuid.uuid4())[9:23]

            # Create body
            body = f"\n--batch_{batch_uuid}\n"
            body += "Content-Type: application/http\n"
            body += "Content-Transfer-Encoding: binary\n\n"
            body += "GET Events?sap-client=200&$skip=0&$top=500&$filter=(date%20ge%20datetime%272021-10-24T00%3a00%3a00%27%20and%20date%20le%20datetime%272021-12-07T00%3a00%3a00%27)&$expand=EventTeacherSet%2cEventLocationSet HTTP/1.1\n"
            body += "sap-cancel-on-close: true\n"
            body += "sap-contextid-accept: header\n"
            body += "Accept: application/json\n"
            body += "Accept-Language: en\n"
            body += "DataServiceVersion: 2.0\n"
            body += "MaxDataServiceVersion: 2.0\n\n"
            body += f"\n--batch_{batch_uuid}--\n"

            # POST
            r = self._SESSION.post(
                url=self._SCHEDULE_URL,
                params={
                    'sap-client': '200'
                },
                headers={
                    'DataServiceVersion': '2.0',
                    'MaxDataServiceVersion': '2.0',
                    'Content-Type': f'multipart/mixed;boundary=batch_{batch_uuid}'
                },
                data=body

            )

            r.raise_for_status()

            data = r.text.splitlines()  # Grab JSON. This is ugly, fix later
            data = data[12:][0]
            schedule = f"[{data}]"
            return json.loads(schedule)

        except KeyError:

            sys.exit(
                f'SCHEDULE: You need an api object with an extended portal session (kuloket) to use this method')

        except Exception as ex:

            sys.exit(f'SCHEDULE: {ex}')


def create_api_object(session: requests.Session) -> ToledoApi:

    return ToledoApi(
        session=session
    )



import sys
import os
import yaml
import requests
import random
import string
import pkce

from toledo.utils import get_dashboard_auth_token
from toledo.utils import get_saml2_relay_and_response
from toledo.utils import get_saml2_csrf

from toledo.utils import post_saml2_csrf
from toledo.utils import post_saml2_relay_and_response

from toledo.utils import get_start_html
from toledo.utils import openid_connect_auth

from toledo.utils import SESSION


class DashboardLogin:

    def __init__(self, portal_session: requests.Session) -> None:

        self._SESSION = portal_session

        try:

            with open(os.path.join(os.path.dirname(__file__), 'config.yaml'), 'r') as f:
                parser = yaml.safe_load(f)

        except FileNotFoundError:

            sys.exit('Unable to find find config.yaml')

        self._DASHBOARDURL = parser['LOGIN']['DashboardURL']
        self._AUTHORIZATION_ENDPOINT = parser['LOGIN']['AuthorizationEndpoint']
        self._TOKEN_ENDPOINT = parser['LOGIN']['TokenEndpoint']

    def __call__(self) -> requests.Session:

        try:
            # 2. Toledo Dashboard

            SESSION = self._SESSION

            if '_shibsession_' not in str(SESSION.cookies.get_dict()):

                raise AttributeError

            # Load Toledo Dashboard
            get_start_html(
                url=self._DASHBOARDURL
            )

            # Generate state
            state = ''.join(random.choices(
                string.ascii_uppercase + string.ascii_lowercase + string.digits, k=32))

            # Generate code_verifier & code_challenge
            code_verifier, code_challenge = pkce.generate_pkce_pair()

            # Authorize using state & code_challenge
            first_csrf_html = openid_connect_auth(
                state=state,
                code_challenge=code_challenge,
                url=self._AUTHORIZATION_ENDPOINT
            )

            # Get first csrf token
            first_csrf = get_saml2_csrf(
                html=first_csrf_html
            )

            # Post first csrf token
            relaystate_samlresponse_html = post_saml2_csrf(
                post_info=first_csrf
            )

            # Get RelayState & SAMLResponse
            relaystate_samlresponse = get_saml2_relay_and_response(
                html=relaystate_samlresponse_html
            )

            # Post RelayState & SAMLResponse and get code
            code = post_saml2_relay_and_response(
                post_info=relaystate_samlresponse,
                return_code=True
            )

            # Post code and code_verifier and get/add auth token
            get_dashboard_auth_token(
                code=code,
                code_verifier=code_verifier,
                url=self._TOKEN_ENDPOINT
            )

            # At this point we have obtained our auth jwt
            return SESSION

        except requests.exceptions.HTTPError as errh:
            sys.exit("Http Error:", errh)
        except requests.exceptions.ConnectionError as errc:
            sys.exit("Error Connecting:", errc)
        except requests.exceptions.Timeout as errt:
            sys.exit("Timeout Error:", errt)
        except requests.exceptions.RequestException as err:
            sys.exit("OOps: Something Else:", err)
        except AttributeError:
            sys.exit(
                'Unable to find _shibsession_ in session. You can only extend portal sessions!')
        except Exception as ex:
            sys.exit(ex)


def extend_session(portal_session: requests.Session) -> requests.Session:

    return DashboardLogin(
        portal_session=portal_session
    )()
